subroutine CalcEnergy(Pos, M, L, rc, r0, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, r0
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 30000d0
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, d, id, id2, id6, id12, shift, rc2
    integer :: i,j

    ! initial
    PEnergy = 0d0
    
    ! cutoff
    Shift = -4d0*(rc**(-12d0) - rc**(-6d0))
    rc2 = rc*rc
    
    do i = 0,NAtom-1
        Posi = Pos(i,:)
        do j = i+1,NAtom-1
           rij = Pos(j,:) - Posi
           rij = rij - L*dnint(rij/L)
           if ( (j.eq.i+1) .and. (mod(j,M).gt.0) ) then
               ! bonded PE
               d = sqrt( sum(rij*rij) )
               PEnergy = PEnergy + 0.5d0*k*(d-r0)*(d-r0)
           else
               ! nonbonded (L-J) PE
               d2 = sum(rij*rij)
               if (d2.gt.rc2) cycle
               id2 = 1d0/d2
               id6 = id2*id2*id2
               id12 = id6*id6
               PEnergy = PEnergy + 4d0*(id12 - id6) + Shift
           end if
        end do
    end do
    
end subroutine


subroutine CalcEnergyForces(Pos, M, L, rc, r0, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, r0
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), dimension(Dim) :: rij, fij, Posi
    real(8), parameter :: k = 30000d0
    real(8) :: d2, d, id, id2, id6, id12, shift, rc2
    integer :: i,j

    ! initial
    PEnergy = 0d0
    Forces = 0d0 

    ! cutoff
    Shift = -4d0*(rc**(-12d0) - rc**(-6d0))
    rc2 = rc*rc
    
    do i = 0,NAtom-1
        Posi = Pos(i,:)
        do j = i+1,NAtom-1
           rij = Pos(j,:) - Posi
           rij = rij - L*dnint(rij/L)
           if ( (j.eq.i+1) .and. (mod(j,M).gt.0) ) then
               ! bonded PE
               d = sqrt( sum(rij*rij) )
               id = 1d0/d
               PEnergy = PEnergy + 0.5d0*k*(d-r0)*(d-r0)
               Fij = rij*k*(d-r0)*id
               Forces(i,:) = Forces(i,:) + Fij
               Forces(j,:) = Forces(j,:) - Fij
           else
               ! nonbonded (L-J) PE
               d2 = sum(rij*rij)
               if (d2.gt.rc2) cycle
               id2 = 1d0/d2
               id6 = id2*id2*id2
               id12 = id6*id6
               PEnergy = PEnergy + 4d0*(id12 - id6) + Shift
               Fij = rij*( (-48d0*id12 + 24d0*id6)*id2 )
               Forces(i,:) = Forces(i,:) + Fij
               Forces(j,:) = Forces(j,:) - Fij
           end if
        end do
    end do

end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, r0, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt, r0
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5d0 * dt*dt * Accel
    Vel = Vel + 0.5d0 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, r0, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5d0 * dt * Accel
    KEnergy = 0.5d0 * sum(Vel*Vel)
end subroutine
